@@ -1,6 +1,6 @@
 %%%-------------------------------------------------------------------
 %%% @author Evgeny Khramtsov <ekhramtsov@process-one.net>
-%%% @copyright (C) 2002-2023 ProcessOne, SARL. All Rights Reserved.
+%%% @copyright (C) 2002-2024 ProcessOne, SARL. All Rights Reserved.
 %%%
 %%% Licensed under the Apache License, Version 2.0 (the "License");
 %%% you may not use this file except in compliance with the License.
@@ -25,6 +25,9 @@
 -export([generate_csr/2]).
 -export([format_error/1]).
 
+%% OTP Application API
+-export([start/2]).
+
 -include_lib("public_key/include/public_key.hrl").
 
 -define(DER_NULL, <<5, 0>>).
@@ -109,10 +112,22 @@
 
 -export_type([error_reason/0, issue_return/0, revoke_return/0, challenge_data/0]).
 
+%%%===================================================================
+%%% OTP Application API
+%%%===================================================================
+
+start(_StartType, _StartArgs) ->
+    application:start(inets),
+    inets:start(httpc, [{profile, ?MODULE}]),
+    httpc:set_options([{ipfamily, inet6fb4}], ?MODULE),
+    {ok, self()}.
+
 %%%===================================================================
 %%% API
 %%%===================================================================
+
 start() ->
+    start(normal, []),
     case application:ensure_all_started(?MODULE) of
 	{ok, _} -> ok;
 	Err -> Err
@@ -602,7 +617,7 @@ http_request(State, ReqFun, RetryTimeout) ->
 			       [{timeout, infinity},
 				{connect_timeout, infinity}],
 			       [{body_format, binary},
-				{sync, false}]) of
+				{sync, false}], ?MODULE) of
 		{ok, Ref} ->
 		    ReqTimeout = min(timer:seconds(10), Timeout),
 		    receive
@@ -612,7 +627,7 @@ http_request(State, ReqFun, RetryTimeout) ->
 			      ReqFun, Response, State, RetryTimeout)
 		    after ReqTimeout ->
 			    ?DEBUG("HTTP request timeout", []),
-			    httpc:cancel_request(Ref),
+			    httpc:cancel_request(Ref, ?MODULE),
 			    http_request(State, ReqFun, RetryTimeout)
 		    end;
 		{error, WTF} ->
@@ -666,7 +681,7 @@ handle_http_response(ReqFun, {{_, Code, Slogan}, Hdrs, Body}, State, RetryTimeou
 	{_, Type} when Type == "application/problem+json";
 		       Type == "application/json" ->
 	    State1 = update_nonce(Hdrs, State),
-	    try jiffy:decode(Body, [return_maps]) of
+	    try json_decode_maps(Body) of
 		JSON when Type == "application/json" ->
 		    {ok, {Code, Hdrs, JSON}, State1};
 		JSON when Type == "application/problem+json" ->
@@ -898,7 +913,7 @@ jose_json(#state{account = {Key, AccURL}, nonce = Nonce} = State, Data, URL) ->
     JwsMap = case AccURL of
 		 undefined ->
 		     {_, BinaryPubKey} = jose_jwk:to_binary(PubKey),
-		     PubKeyJson = jiffy:decode(BinaryPubKey),
+		     PubKeyJson = json_decode_maps(BinaryPubKey),
 		     JwsMap0#{<<"jwk">> => PubKeyJson};
 		 _ ->
 		     JwsMap0#{<<"kid">> => iolist_to_binary(AccURL)}
@@ -915,7 +930,19 @@ auth_key(#state{account = {PrivKey, _}}, Token) ->
 
 -spec encode_json(map()) -> binary().
 encode_json(JSON) ->
-    iolist_to_binary(jiffy:encode(JSON)).
+    json_encode(JSON).
+
+-ifdef(OTP_BELOW_27).
+json_encode(Term) ->
+    iolist_to_binary(jiffy:encode(Term)).
+json_decode_maps(Bin) ->
+    jiffy:decode(Bin, [return_maps]).
+-else.
+json_encode(Term) ->
+    iolist_to_binary(json:encode(Term)).
+json_decode_maps(Bin) ->
+    json:decode(Bin).
+-endif.
 
 %%%===================================================================
 %%% Misc
